import {
  Tree
} from "./chunk-B3QRDH5K.js";
import {
  split,
  splitKey
} from "./chunk-4ENVCU5M.js";

// src/Operator.ts
var Operator = class {
  constructor(treeKind, delimiter = "/") {
    this.treeKind = treeKind;
    this.delimiter = delimiter;
    this.tree = new Tree(treeKind, ".");
  }
  tree;
  setToRootOrParent(component, currentPathName, previousPathArray, nextPathArray) {
    const childComponent = this.tree.getChildByPaths(nextPathArray, component.kind);
    if (childComponent || nextPathArray.length === 0) {
      return;
    }
    if (previousPathArray.length === 0 && nextPathArray.length === 1) {
      this.tree.set(nextPathArray[0], component);
    } else {
      const parentComponent = this.tree.getChildByPaths(previousPathArray, this.treeKind);
      parentComponent && parentComponent.set(currentPathName, component);
    }
  }
  removeFromRootOrParent(currentPathName, currentPathArray, component) {
    const previousPathArray = currentPathArray.slice(0, currentPathArray.length - 1);
    if (previousPathArray.length === 0 && currentPathArray.length === 1) {
      this.tree.remove(currentPathName, component);
    } else {
      const parentComponent = this.tree.getChildByPaths(previousPathArray, this.treeKind);
      parentComponent && parentComponent.remove(currentPathName, component);
    }
  }
  getHierarchy() {
    return this.tree.getHierarchy();
  }
  getChildByPaths(path, kind) {
    const pathArray = split(path, this.delimiter);
    return this.tree.getChildByPaths(pathArray, kind);
  }
  set(path, component) {
    const pathArray = split(path, this.delimiter);
    const pathArrayLength = pathArray.length;
    pathArray.reduce((previousPathArray, currentPathName, currentIndex) => {
      const nextPathArray = previousPathArray.concat(currentPathName);
      const isLastIndex = currentIndex === pathArrayLength - 1;
      if (isLastIndex) {
        this.setToRootOrParent(component, currentPathName, previousPathArray, nextPathArray);
      } else {
        const tree = new Tree(this.treeKind, currentPathName);
        this.setToRootOrParent(tree, currentPathName, previousPathArray, nextPathArray);
      }
      return nextPathArray;
    }, []);
  }
  remove(path, kind) {
    const pathArray = split(path, this.delimiter);
    const pathArrayLength = pathArray.length;
    for (let i = 0; i <= pathArrayLength; i++) {
      const currentPathArray = pathArray.slice(0, pathArrayLength - i);
      const currentPathName = pathArray[pathArrayLength - i - 1];
      if (!currentPathName) {
        continue;
      }
      if (i === 0) {
        const component = this.tree.getChildByPaths(currentPathArray, kind);
        component && this.removeFromRootOrParent(currentPathName, currentPathArray, component);
      } else {
        const component = this.tree.getChildByPaths(currentPathArray, this.treeKind);
        component && component.hasChildren() && this.removeFromRootOrParent(currentPathName, currentPathArray, component);
      }
    }
  }
  copy(from, to, kind) {
    const fromComponent = this.getChildByPaths(from, kind);
    const toComponent = this.getChildByPaths(to, kind);
    if (toComponent || !fromComponent) {
      return false;
    }
    this.set(to, fromComponent);
    return true;
  }
  move(from, to, kind) {
    const success = this.copy(from, to, kind);
    if (success) {
      this.remove(from, kind);
      return true;
    }
    return false;
  }
  getChildPaths(kind, children, parentPath) {
    let pathArray = [];
    Object.entries(children).forEach(([key, child]) => {
      const subChildren = child.getChildren();
      const [, pathName] = splitKey(key);
      const nextPath = [parentPath, pathName].join(this.delimiter).replace(/\/$/, "");
      if (subChildren) {
        pathArray = pathArray.concat(this.getChildPaths(kind, subChildren, nextPath));
      } else if (child.kind === kind) {
        pathArray.push(nextPath);
      }
    });
    return pathArray;
  }
  getNodePaths(kind) {
    return this.getChildPaths(kind, this.tree.getChildren(), this.tree.name);
  }
};

export {
  Operator
};
//# sourceMappingURL=chunk-FR6AL2AE.js.map