"use strict";
var __defProp = Object.defineProperty;
var __getOwnPropDesc = Object.getOwnPropertyDescriptor;
var __getOwnPropNames = Object.getOwnPropertyNames;
var __hasOwnProp = Object.prototype.hasOwnProperty;
var __export = (target, all) => {
  for (var name in all)
    __defProp(target, name, { get: all[name], enumerable: true });
};
var __copyProps = (to, from, except, desc) => {
  if (from && typeof from === "object" || typeof from === "function") {
    for (let key of __getOwnPropNames(from))
      if (!__hasOwnProp.call(to, key) && key !== except)
        __defProp(to, key, { get: () => from[key], enumerable: !(desc = __getOwnPropDesc(from, key)) || desc.enumerable });
  }
  return to;
};
var __toCommonJS = (mod) => __copyProps(__defProp({}, "__esModule", { value: true }), mod);

// src/index.ts
var src_exports = {};
__export(src_exports, {
  Node: () => Node,
  Operator: () => Operator,
  Tree: () => Tree,
  Utils: () => Utils_exports
});
module.exports = __toCommonJS(src_exports);

// src/Node.ts
var Node = class {
  constructor(kind, name) {
    this.kind = kind;
    this.name = name;
  }
  getChildren() {
    return void 0;
  }
  hasChildren() {
    return false;
  }
  getHierarchy() {
    return {
      name: this.name
    };
  }
  getChildByPaths() {
    return void 0;
  }
  set() {
    return;
  }
  remove() {
    return;
  }
};

// src/Utils.ts
var Utils_exports = {};
__export(Utils_exports, {
  generateKey: () => generateKey,
  split: () => split,
  splitKey: () => splitKey
});
var generateKey = (kind, name) => {
  return `${kind}:${name}`;
};
var splitKey = (key) => {
  return key.split(":");
};
var split = (p, delimiter) => {
  const array = [];
  for (const seg of p.split(delimiter)) {
    switch (seg) {
      case ".":
        break;
      case "..":
        array.pop();
        break;
      default:
        array.push(seg);
        break;
    }
  }
  return array;
};

// src/Tree.ts
var Tree = class {
  constructor(kind, name) {
    this.kind = kind;
    this.name = name;
  }
  children = {};
  getChildByPaths(paths, kind) {
    const [name, ...pathArray] = paths;
    if (!name) {
      return;
    }
    if (pathArray.length === 0) {
      const component = this.children[generateKey(kind, name)];
      return component;
    }
    const childTree = this.children[generateKey(this.kind, name)];
    return childTree && childTree.getChildByPaths(pathArray, kind);
  }
  getHierarchy() {
    const entires = Object.entries(this.children).map(([key, child]) => {
      return [key, child.getHierarchy()];
    });
    return {
      name: this.name,
      children: Object.fromEntries(entires)
    };
  }
  getChildren() {
    return this.children;
  }
  hasChildren() {
    return Object.keys(this.children).length === 0;
  }
  set(pathName, component) {
    const key = generateKey(component.kind, pathName);
    this.children[key] = component;
  }
  remove(pathName, component) {
    const targetKey = generateKey(component.kind, pathName);
    const entries = Object.entries(this.children).filter(([key]) => {
      return targetKey !== key;
    });
    this.children = Object.fromEntries(entries);
  }
};

// src/Operator.ts
var Operator = class {
  constructor(treeKind, delimiter = "/") {
    this.treeKind = treeKind;
    this.delimiter = delimiter;
    this.tree = new Tree(treeKind, ".");
  }
  tree;
  setToRootOrParent(component, currentPathName, previousPathArray, nextPathArray) {
    const childComponent = this.tree.getChildByPaths(nextPathArray, component.kind);
    if (childComponent || nextPathArray.length === 0) {
      return;
    }
    if (previousPathArray.length === 0 && nextPathArray.length === 1) {
      this.tree.set(nextPathArray[0], component);
    } else {
      const parentComponent = this.tree.getChildByPaths(previousPathArray, this.treeKind);
      parentComponent && parentComponent.set(currentPathName, component);
    }
  }
  removeFromRootOrParent(currentPathName, currentPathArray, component) {
    const previousPathArray = currentPathArray.slice(0, currentPathArray.length - 1);
    if (previousPathArray.length === 0 && currentPathArray.length === 1) {
      this.tree.remove(currentPathName, component);
    } else {
      const parentComponent = this.tree.getChildByPaths(previousPathArray, this.treeKind);
      parentComponent && parentComponent.remove(currentPathName, component);
    }
  }
  getHierarchy() {
    return this.tree.getHierarchy();
  }
  getChildByPaths(path, kind) {
    const pathArray = split(path, this.delimiter);
    return this.tree.getChildByPaths(pathArray, kind);
  }
  set(path, component) {
    const pathArray = split(path, this.delimiter);
    const pathArrayLength = pathArray.length;
    pathArray.reduce((previousPathArray, currentPathName, currentIndex) => {
      const nextPathArray = previousPathArray.concat(currentPathName);
      const isLastIndex = currentIndex === pathArrayLength - 1;
      if (isLastIndex) {
        this.setToRootOrParent(component, currentPathName, previousPathArray, nextPathArray);
      } else {
        const tree = new Tree(this.treeKind, currentPathName);
        this.setToRootOrParent(tree, currentPathName, previousPathArray, nextPathArray);
      }
      return nextPathArray;
    }, []);
  }
  remove(path, kind) {
    const pathArray = split(path, this.delimiter);
    const pathArrayLength = pathArray.length;
    for (let i = 0; i <= pathArrayLength; i++) {
      const currentPathArray = pathArray.slice(0, pathArrayLength - i);
      const currentPathName = pathArray[pathArrayLength - i - 1];
      if (!currentPathName) {
        continue;
      }
      if (i === 0) {
        const component = this.tree.getChildByPaths(currentPathArray, kind);
        component && this.removeFromRootOrParent(currentPathName, currentPathArray, component);
      } else {
        const component = this.tree.getChildByPaths(currentPathArray, this.treeKind);
        component && component.hasChildren() && this.removeFromRootOrParent(currentPathName, currentPathArray, component);
      }
    }
  }
  copy(from, to, kind) {
    const fromComponent = this.getChildByPaths(from, kind);
    const toComponent = this.getChildByPaths(to, kind);
    if (toComponent || !fromComponent) {
      return false;
    }
    this.set(to, fromComponent);
    return true;
  }
  move(from, to, kind) {
    const success = this.copy(from, to, kind);
    if (success) {
      this.remove(from, kind);
      return true;
    }
    return false;
  }
  getChildPaths(kind, children, parentPath) {
    let pathArray = [];
    Object.entries(children).forEach(([key, child]) => {
      const subChildren = child.getChildren();
      const [, pathName] = splitKey(key);
      const nextPath = [parentPath, pathName].join(this.delimiter).replace(/\/$/, "");
      if (subChildren) {
        pathArray = pathArray.concat(this.getChildPaths(kind, subChildren, nextPath));
      } else if (child.kind === kind) {
        pathArray.push(nextPath);
      }
    });
    return pathArray;
  }
  getNodePaths(kind) {
    return this.getChildPaths(kind, this.tree.getChildren(), this.tree.name);
  }
};
// Annotate the CommonJS export names for ESM import in node:
0 && (module.exports = {
  Node,
  Operator,
  Tree,
  Utils
});
//# sourceMappingURL=index.cjs.map